package cn.ninegame.genericframework.config;

import android.content.Context;

import java.io.File;

import cn.ninegame.genericframework.tools.endecode.Md5Utils;

import static cn.ninegame.genericframework.config.DynamicLoadConfig.MODULE_PRIVATE_ROOT_PATH;

/**
 * ****************************************************************************
 * Copyright @ 2009 - 2015 www.9game.cn All Rights Reserved
 * <p/>
 * Creation    : 2016-02-03
 * Author      : lihq@ucweb.com
 * Description : 管理dex加载的各种路径
 *
 * --data/data/appname
 *   |
 *   |---modules
 *          |---modulePackageName
 *                     |---data
 *                     |      |---signature
 *                     |---version1.0.0
 *                     |      |---'lib' + modulePackageName.apk   (此文件伪装成so放在libs目录，系统不支持加载so格式的apk才会有此文件)
 *                     |      |---manifest.json
 *                     |      |---property
 *                     |      |---code_cache
 *                     |      |       |---'lib' + modulePackageName.dex
 *                     |      |       |---module_odex_finish
 *                     |      |---so
 *                     |---version2
 * ****************************************************************************
 */
public class ModulePathManager {

    Context mContext;
    private static ModulePathManager gInstance;
    private File mApkFile;
    private File mRootApkFile;


    public ModulePathManager() {}

    public static ModulePathManager getInstance(){
        if(gInstance == null){
            synchronized (ModulePathManager.class){
                if(gInstance == null){
                    gInstance = new ModulePathManager();
                }
            }
        }

        return gInstance;
    }

    /**
     *
     * @param context
     * @param apkFile  framework所在apk的当前路径
     */
    public void init(Context context, File apkFile){
        mContext = context;
        mApkFile = apkFile;
        if(!context.getApplicationContext().getPackageCodePath().equals(context.getPackageCodePath())){
            mRootApkFile = new File(context.getApplicationContext().getPackageCodePath());
        }
    }

    /**
     *
     * @param context
     * @param apkFile  framework所在apk的当前路径
     */
    public void init(Context context, File apkFile,String instanceName){
        DynamicLoadConfig.MODULE_PRIVATE_ROOT_PATH = instanceName+"_modules";
        mContext = context;
        mApkFile = apkFile;
        if(!context.getApplicationContext().getPackageCodePath().equals(context.getPackageCodePath())){
            mRootApkFile = new File(context.getApplicationContext().getPackageCodePath());
        }
    }


    /**
     * 获取module在assets的路径
     * @param moduleID
     * @return
     */
    public String getModuleAssetsPath(String moduleID){
        return DynamicLoadConfig.MODULE_ASSETS_ROOT_PATH + File.separator + moduleID + ".apk";
    }

    /**
     * 返回data/data下module的根目录
     * @param moduleID
     * @return
     */
    public String getModuleRootDirPath(String moduleID){
        String path =  mContext.getApplicationInfo().dataDir+ File.separator + MODULE_PRIVATE_ROOT_PATH + File.separator+ moduleID;
        return path;
    }

    /**
     * 返回data/data/modules根目录
     * @return
     */
    public String getRootPath(){
        return mContext.getApplicationInfo().dataDir+ File.separator + MODULE_PRIVATE_ROOT_PATH;
    }

    public String getAppRootPath(){
        return mContext.getApplicationInfo().dataDir;
    }

    public File getModuleRootDir(String moduleID){
        String rootDirPath = getModuleRootDirPath(moduleID);
        return new File(rootDirPath);
    }

    /**
     * 获取给定version的目录
     * @param moduleID
     * @param version
     * @return
     */
    public File getModuleVersionDir(String moduleID, String version){
        String rootDirPath = getModuleRootDirPath(moduleID);
        return new File(rootDirPath,version);
    }

    /**
     * 获取指定version path
     * @param moduleID
     * @param version
     * @return
     */
    public String getModuleVersionPath(String moduleID, String version){
        String rootDirPath = getModuleRootDirPath(moduleID);
        return rootDirPath + File.separator + version;
    }

    /**
     * 返回指定module在某个version下的apk
     * like: data/data/appname/modules/modulePackageName/version1/modulePackageName.apk
     * @param moduleID
     * @param version
     * @return
     */
    public File getModuleApk(String moduleID, String version){
        return new File(getModuleApkPath(moduleID, version));
    }

    /**
     * 返回指定module在某个version下的apk路径
     * like: data/data/appname/modules/modulePackageName/version1/modulePackageName.apk
     * @param moduleID
     * @param version
     * @return
     */
    public String getModuleApkPath(String moduleID, String version) {
        return new StringBuffer()
                .append(getModuleVersionPath(moduleID, version))
                .append(File.separator)
                .append("lib")
                .append(moduleID)
                .append(DynamicLoadConfig.MODULE_NAME_SUFFIX_IN_MODULE)
                .toString();
    }

    /**
     * 返回Lib目录下的module apk文件
     * like: data/data/packageName/lib/"lib"+modulePackageName+".so"
     * @param moduleID
     * @return
     */
    public File getModuleApkInLib(String moduleID){
        return new File(getModuleApkPathInLib(moduleID));
    }

    public File getModuleApkInAssets(String moduleID){
        return new File(getModuleApkPathInLib(moduleID));
    }

    /**
     * 返回Lib目录下的module apk文件路径，
     * data/data/packageName/lib/"lib"+modulePackageName+".so"
     * 例: data/data/packageName/lib/libcn.ninegame.floating.so
     * @param moduleID
     * @return
     */
    public String getModuleApkPathInLib(String moduleID){
        return new StringBuffer()
                .append(getAppRootPath())
                .append(File.separator)
                .append("lib")
                .append(File.separator)
                .append(DynamicLoadConfig.MODULE_NAME_PREFIX)
                .append(moduleID)
                .append(DynamicLoadConfig.MODULE_NAME_SUFFIX)
                .toString();
    }

    /**
     * 返回asset目录下的module apk文件路径，
     * like: data/data/appname/modules/modulePackageName/version1/modulePackageName.apk
     * @param moduleID
     * @param version
     * @return
     */
    public String getModuleApkPathInAssets(String moduleID, String version) {
        return new StringBuffer()
                .append(getModuleVersionPath(moduleID, version))
                .append(File.separator)
                .append("lib")
                .append(moduleID)
                .append(DynamicLoadConfig.MODULE_NAME_SUFFIX_IN_MODULE)
                .toString();
    }

    /**
     * 获取 module apk 在主apk文件中的全路径
     * 例: /lib/armeabi-v7a/libcn.ninegame.floating.so
     * @return
     */
    public String getModuleApkAbsolutePathInLib(String moduleID){
        return new StringBuffer()
                .append("lib/armeabi-v7a/")
                .append(DynamicLoadConfig.MODULE_NAME_PREFIX)
                .append(moduleID)
                .append(DynamicLoadConfig.MODULE_NAME_SUFFIX)
                .toString();
    }

    /**
     * 获取 module apk 在主apk文件中的全路径
     * 例: /lib/armeabi/libcn.ninegame.floating.so
     * @return
     */
    public String getModuleApkAbsolutePath(String moduleID){
        return new StringBuffer()
                .append("assets/lib/armeabi/")
                .append(DynamicLoadConfig.MODULE_NAME_PREFIX)
                .append(moduleID)
                .append(DynamicLoadConfig.MODULE_NAME_SUFFIX)
                .toString();
    }

    /**
     * 在Flashbuild模式中获取 module apk
     * 例: /data/local/tmp/gundam/modules/cn.ninegame.account/libcn.ninegame.account.so
     * @return
     */
    public File getModuleApkInFlashbuild(String moduleID) {
        String moduleApkPath = new StringBuilder()
                .append(DynamicLoadConfig.MODULE_FLASHBUILD_ROOT)
                .append(moduleID)
                .append(File.separator)
                .append(DynamicLoadConfig.MODULE_NAME_PREFIX)
                .append(moduleID)
                .append(DynamicLoadConfig.MODULE_NAME_SUFFIX)
                .toString();

        return new File(moduleApkPath);
    }

    /**
     * 在Flashbuild模式中获取 module apk
     * 例: /data/local/tmp/gundam/modules/cn.ninegame.account/ModuleManifest.json
     * @return
     */
    public File getModuleManifestInFlashbuild(String moduleID) {
        String moduleApkPath = new StringBuilder()
                .append(DynamicLoadConfig.MODULE_FLASHBUILD_ROOT)
                .append(moduleID)
                .append(File.separator)
                .append(DynamicLoadConfig.MODULE_MANIFEST_FILE)
                .toString();

        return new File(moduleApkPath);
    }

    /**
     * 返回module解压出来的manifest
     * @param moduleID
     * @param version
     * @return
     */
    public File getModuleManifest(String moduleID, String version){
        String versionPath = getModuleVersionPath(moduleID, version);
        return new File(versionPath, DynamicLoadConfig.MODULE_MANIFEST_FILE);
    }

    /**
     * 返回module odex存放目录
     * @param moduleID
     * @param version
     * @return
     */
    public File getModuleOdexDir(String moduleID, String version){
        String versionPath = getModuleVersionPath(moduleID, version);
        return new File(versionPath,DynamicLoadConfig.MODULE_CODE_CACHE);
    }

    public File getModuleOdex(String moduleID, String version){
        String versionPath = getModuleVersionPath(moduleID, version);
        String file = new StringBuffer()
                .append(DynamicLoadConfig.MODULE_CODE_CACHE)
                .append(File.separator)
                .append(DynamicLoadConfig.MODULE_NAME_PREFIX)
                .append(moduleID)
                .append(DynamicLoadConfig.MODULE_ODEX_FILE_SUFFIX)
                .toString();
        return new File(versionPath, file);
    }

    public File getModuleOdexFinish(String moduleID, String version){
        String versionPath = getModuleVersionPath(moduleID, version);
        String file = new StringBuffer()
                .append(DynamicLoadConfig.MODULE_CODE_CACHE)
                .append(File.separator)
                .append(DynamicLoadConfig.MODULE_ODEX_FINISH)
                .toString();
        return new File(versionPath, file);
    }

    /**
     * 返回module data目录
     * @param moduleID
     * @return
     */
    public File getModuleDataDir(String moduleID){
        return new File(getModuleRootDirPath(moduleID), DynamicLoadConfig.MODULE_DATA);
    }

    /**
     * 返回module data目录
     * @param moduleID
     * @return
     */
    public String getModuleDataDirPath(String moduleID){
        return getModuleRootDirPath(moduleID) + File.separator + DynamicLoadConfig.MODULE_DATA;
    }

    /**
     * 返回so目录
     * @param moduleID
     * @param version
     * @return
     */
    public File getModuleSODir(String moduleID, String version){
        String versionPath = getModuleVersionPath(moduleID, version);
        return new File(versionPath, DynamicLoadConfig.MODULE_SO);
    }

    /**
     * 返回so目录
     * @param moduleID
     * @param version
     * @return
     */
    public String getModuleSODirPath(String moduleID, String version){
        String versionPath = getModuleVersionPath(moduleID, version);
        return versionPath + File.separator + DynamicLoadConfig.MODULE_SO;
    }

    /**
     * 主manifest
     * @return
     */
    public File getMainManifestFile(){
        return new File(getRootPath(),DynamicLoadConfig.MODULE_MANIFEST_FILE);
    }

    /**
     * 获取module配置文件名（module.json）
     * @return
     */
    public String getManifestFileName(){
        return DynamicLoadConfig.MODULE_MANIFEST_FILE;
    }

    /**
     * module 签名文件路径
     */
    public String getModuleSignaturePath(String moduleID){
        return getModuleDataDirPath(moduleID) + File.separator + DynamicLoadConfig.MODULE_SIGNATURE;
    }

    /**
     * module 签名文件
     */
    public File getModuleSignatureFile(String moduleID){
        return new File(getModuleSignaturePath(moduleID));
    }

    public String getModuleLockPath(Context context, String moduleID){
        return context.getFilesDir() + File.separator + Md5Utils.getMD5(moduleID) + DynamicLoadConfig.MODULE_LOCK_SUFFIX;
    }

    public File getModulePropertyFile(String moduleID, String version){
        return new File(new StringBuffer().append(getModuleVersionPath(moduleID, version))
                .append(File.separator).append(DynamicLoadConfig.MODULE_PROPERTY)
                .toString());
    }

    public File getSelfApkFile() {
        return mApkFile;
    }

    public File getSelfRootApkFile() {
        if(mRootApkFile!=null){
            return mRootApkFile;
        }
        return mApkFile;
    }
}
