package cn.ninegame.genericframework.module;

import java.lang.reflect.Method;

import android.app.Application;
import android.content.Context;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.content.res.AssetManager;
import android.content.res.Resources;
import android.os.Build;
import android.support.annotation.Nullable;
import cn.ninegame.genericframework.exception.LoadContextException;
import cn.ninegame.genericframework.tools.ReflectHelper;

/**
 * 创建自定义Context与Resource
 *
 * @author:ligs@ucweb.com
 */
public class ModuleResource {

    private Context mApplication;
    private Context mContext;
    private Resources mResources;
    private ModuleInfo mModuleInfo;

    public ModuleResource(Application application, ModuleInfo pInfo) {
        this.mModuleInfo = pInfo;
        this.mApplication = application;
    }

    public Context createContext() throws LoadContextException {
        try {
            if (null == this.mResources)
                createResources();

            String appPackageName = mApplication.getPackageName();
            // createPackageContext 中的包名必须为已经安装的包名，此处实际上是从原应用的context中复制一个出来
            // 需要说明的是通过createPackageContext中创建的Context，最后都会通过ActivityThread中的getPackageInfo，这方法会缓存已经创建过的同名LoadedApk(包名为key）
            Context ctx = mApplication.createPackageContext(appPackageName, Context.CONTEXT_RESTRICTED);

            //为当前Plugin创建一个LoadedApk
            Object loadedApk = createLoadedApk(appPackageName);

            if (loadedApk != null) {
                //使用已经创建好的resources对象添加到ContextImpl中的mResources
                ReflectHelper.setField(ctx, "mResources", mResources);
                ReflectHelper.setField(ctx, "mPackageInfo", loadedApk);
                ReflectHelper.setField(ctx, "mPackageInfo.mApplication", this.mApplication);
                //FIXME by zhonghui, wegame, ingnor it!!!!!!!!
                mContext = new ModuleContext(mApplication, ctx.getResources());
            }
        } catch (Exception e) {
            throw new LoadContextException("Could not create context from " + mModuleInfo.getID(), e);
        }

        return this.mContext;
    }

    private Resources createResources() throws LoadContextException {
        String apkFilePath = "";
        if (null == this.mResources) {
            try {
                apkFilePath = this.mModuleInfo.getSourceFile().getAbsolutePath();
                String filePath = apkFilePath;
                AssetManager assetManager = AssetManager.class.newInstance();
                Method addAssetPath = assetManager.getClass().getMethod("addAssetPath", String.class);
                addAssetPath.invoke(assetManager, (Object) filePath);
                Resources appResources = mApplication.getResources();
                mResources = new Resources(assetManager, appResources.getDisplayMetrics(), appResources.getConfiguration());
            } catch (Exception e) {
                throw new LoadContextException("Could not create resources from " + mModuleInfo.getID(), e);
            }
        }
        return this.mResources;
    }

    /*
     * 创建一个插件的LoadedApk对象，需要应用包名（packageName）与插件包名(pluginPackageName)
     */
    private Object createLoadedApk(String packageName) throws LoadContextException {
        try {
            Context baseCtx = ReflectHelper.getFieldValue(this.mApplication, "mBase");//获取Base Context对象
            Object mainThread = ReflectHelper.getFieldValue(baseCtx, "mMainThread");//获取ActivityThread对象

            if (mainThread != null) {
                //获取宿主的ApplicationInfo
                ApplicationInfo baseAppInfo = baseCtx.getPackageManager().getApplicationInfo(packageName, PackageManager.GET_SHARED_LIBRARY_FILES | PackageManager.GET_ACTIVITIES);
                //获取插件的ApplicationInfo
                ApplicationInfo plgAppInfo = this.mModuleInfo.getPackageInfo(baseCtx).applicationInfo;
                plgAppInfo.uid = baseAppInfo.uid;
                plgAppInfo.sourceDir = this.mModuleInfo.getSourceFile().getAbsolutePath();
                plgAppInfo.publicSourceDir = plgAppInfo.sourceDir;
                plgAppInfo.processName = baseAppInfo.processName;//process设置成与宿主一样
                plgAppInfo.dataDir = baseAppInfo.dataDir;//data的空间路径与宿主一样 
                plgAppInfo.taskAffinity = baseAppInfo.taskAffinity;//task栈与宿主一致

                Object loadedApk = null;
                if (Build.VERSION.SDK_INT > Build.VERSION_CODES.GINGERBREAD_MR1) {
                    //get CompatibilityInfo
                    Method getCompatibilityInfoMethod = this.mResources.getClass().getDeclaredMethod("getCompatibilityInfo", new Class[]{});
                    Object compatibilityObj = getCompatibilityInfoMethod.invoke(this.mResources, new Object[]{});

                    ClassLoader classLoader = this.getClass().getClassLoader();
                    if (classLoader != null) {
                        Class<?> compatibilityInfoClass = classLoader.loadClass("android.content.res.CompatibilityInfo");
                        Method getPackageInfoMethod = mainThread.getClass().getDeclaredMethod("getPackageInfo", new Class[]{ApplicationInfo.class, compatibilityInfoClass, int.class});
                        loadedApk = getPackageInfoMethod.invoke(mainThread, new Object[]{plgAppInfo, compatibilityObj, Context.CONTEXT_INCLUDE_CODE | Context.CONTEXT_IGNORE_SECURITY});
                    }
                } else {
                    Method getPackageInfoMethod = mainThread.getClass().getDeclaredMethod("getPackageInfo", new Class[]{ApplicationInfo.class, int.class});
                    loadedApk = getPackageInfoMethod.invoke(mainThread, new Object[]{plgAppInfo, Context.CONTEXT_INCLUDE_CODE | Context.CONTEXT_IGNORE_SECURITY});
                }

                //set ClassLoader
                ReflectHelper.setField(loadedApk, "mClassLoader", this.getClass().getClassLoader());
                ReflectHelper.setField(loadedApk, "mResources", this.mResources);
                return loadedApk;
            }
        } catch (Exception e) {
            throw new LoadContextException("Could not create application from " + mModuleInfo.getID(), e);
        }
        return null;
    }
}
