package cn.ninegame.genericframework.module.flashbuild;

import android.content.Context;
import android.os.Bundle;
import android.text.TextUtils;

import com.alibaba.fastjson.JSON;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;

import cn.ninegame.genericframework.basic.FrameworkFacade;
import cn.ninegame.genericframework.basic.FrameworkMessage;
import cn.ninegame.genericframework.config.DynamicLoadConfig;
import cn.ninegame.genericframework.config.ModulePathManager;
import cn.ninegame.genericframework.module.JsonModuleManifestParser;
import cn.ninegame.genericframework.module.Module;
import cn.ninegame.genericframework.module.ModuleConfig;
import cn.ninegame.genericframework.module.ModuleLoader;
import cn.ninegame.genericframework.module.ModuleSignature;
import cn.ninegame.genericframework.tools.FileUtil;
import cn.ninegame.genericframework.tools.LOG;
import cn.ninegame.genericframework.tools.ZipUtil;
import cn.ninegame.genericframework.tools.dynamicload.DexLoader;
import dalvik.system.DexFile;

/**
 * Created by lingcong on 2016/11/29.
 */
public class FlashbuildInstaller {
    private Context mContext = null;
    private String mModuleID = null;

    private static final String TAG = "Flashbuild";

    public FlashbuildInstaller(Context context,String moduleID) {
        mContext = context;
        mModuleID = moduleID;
    }

    public void install() {
        boolean bResult = false;
        String errorMsg = null;
        String moduleID = null;
        String moduleVerision = null;
        if (TextUtils.isEmpty(mModuleID)) {
            return;
        }
        try {
            //获取flashbuild目录下的Module APK。
            File flashbuildAPK = ModulePathManager.getInstance().getModuleApkInFlashbuild(mModuleID);
            if (!flashbuildAPK.exists()) {
                errorMsg = "Flashbuild module APK is null ,module id is " + mModuleID;
                throw new IllegalStateException();
            }

            //获取flashbuild目录下的ModuleManifest.json
            Module flashbuildModule = parseModuleManifest();
            if (flashbuildModule == null) {
                errorMsg = "Flashbuild moduleManifest is null ,module id is " + mModuleID;
                throw new IllegalStateException();
            }
            moduleID = flashbuildModule.getId();
            moduleVerision = flashbuildModule.getVersion();

            //删除原有的version下的数据
            File versionDir = ModulePathManager.getInstance().getModuleVersionDir(mModuleID,flashbuildModule.getVersion());
            if (versionDir.exists()) {
                FileUtil.deleteDirectory(versionDir);
            }

            if (versionDir.exists()) {
                //删除失败，本次flashbuild失败。
                errorMsg = "Flashbuild delete old version data error, module id is " +mModuleID;
                throw new IllegalStateException();
            }

            //重新创建version dir，将flashbuild的apk拷贝进去
            if(!versionDir.mkdirs()) {
                errorMsg = "Flashbuild make version dir error, module id is " +mModuleID;
                throw new IllegalStateException();
            }
            String targetName = flashbuildAPK.getName().replace(DynamicLoadConfig.MODULE_NAME_SUFFIX,DynamicLoadConfig.MODULE_NAME_SUFFIX_IN_MODULE);
            FileUtil.copy(flashbuildAPK,versionDir,targetName);
            File apk = ModulePathManager.getInstance().getModuleApk(mModuleID,flashbuildModule.getVersion());
            if (!apk.exists()) {
                throw new IllegalStateException();
            }

            //flalshbuild APK 进行ODEX，并把odex文件copy到module目录
            boolean bOdexResult = makeOdex(flashbuildAPK,flashbuildModule.getVersion());
            if (!bOdexResult) {
                errorMsg = "Flashbuild makeODEX is failed ,module id is " + mModuleID;
                throw new IllegalStateException();
            }

            //生成property文件，并将加载路径指向module dir
            File propertyFile = ModulePathManager.getInstance().getModulePropertyFile(mModuleID, flashbuildModule.getVersion());
            try {
                propertyFile.createNewFile();
            } catch (IOException e) {
                e.printStackTrace();
            }
            FileUtil.writeProperty(propertyFile,ModuleLoader.MODULE_APK_DIR,String.valueOf(ModuleLoader.DIR_MODULE));

            ArrayList<Module> currentModules = parseMainModuleManifest();
            if (currentModules == null) {
                errorMsg = "Flashbuild MainModuleManifest is null";
                throw new IllegalStateException();
            }

            int i = 0;
            for (; i < currentModules.size(); i++) {
                Module oldModule = currentModules.get(i);
                if (flashbuildModule.getId().equals(oldModule.getId())) {
                    currentModules.set(i, flashbuildModule);
                    break;
                }
            }

            if (i == currentModules.size()) {
                errorMsg = "Flashbuild: can't find target module,module id is " + mModuleID;
                throw new IllegalStateException();
            }

            //  更新模块签名，避免被当成覆盖安装
            new ModuleSignature(flashbuildModule.getId()).updateSignature(flashbuildModule.getSignature());

            //模块总表落地
            ModuleConfig moduleManifest = new ModuleConfig();
            moduleManifest.setModules(currentModules);
            byte[] manifestStr = JSON.toJSONBytes(moduleManifest);
            bResult = FileUtil.writeFile(ModulePathManager.getInstance().getMainManifestFile().getPath(), manifestStr);
        }catch (Exception e) {
        }

        if (!bResult) {
            rollbackFlashbuild(moduleVerision);
        }

        Bundle bundle = new Bundle();
        bundle.putBoolean(FrameworkMessage.KEY_FLASH_BUILD_RESULT,bResult);
        bundle.putString(FrameworkMessage.KEY_FLASH_BUILD_ERROR_MSG,errorMsg);
        bundle.putString(FrameworkMessage.KEY_FLASH_BUILD_MODULE,moduleID);
        FrameworkFacade.getInstance().getEnvironment().sendMessage(FrameworkMessage.FLASH_BUILD_COMPLETE,bundle);

        return;
    }

    private void rollbackFlashbuild(String version) {
        if (TextUtils.isEmpty(version)) {
            return;
        }
        File apk = ModulePathManager.getInstance().getModuleApk(mModuleID,version);
        if (apk.exists()) {
            apk.delete();
        }

        File odexDir = ModulePathManager.getInstance().getModuleOdexDir(mModuleID,version);
        if (odexDir.exists()) {
            FileUtil.deleteDirectory(odexDir);
        }

        File propertyFile = ModulePathManager.getInstance().getModulePropertyFile(mModuleID, version);
        if (propertyFile.exists()) {
            propertyFile.delete();
        }
    }


    /**
     * 解析现有模块总表
     * @return
     */
    private ArrayList<Module> parseMainModuleManifest(){
        LOG.d(TAG, "parserMainModuleManifest");
        ArrayList<Module> currentModules = null;
        File file = ModulePathManager.getInstance().getMainManifestFile();

        if (file != null && file.exists()) {
            try {
                currentModules = new JsonModuleManifestParser().parserForModule(new FileInputStream(file));
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        return currentModules;
    }

    private Module parseModuleManifest() {
        Module module = null;
        File manifestFile = ModulePathManager.getInstance().getModuleManifestInFlashbuild(mModuleID);
        if (manifestFile == null && !manifestFile.exists()) {
            return null;
        }

        ModuleConfig moduleManifest = JSON.parseObject(FileUtil.fileToByte(manifestFile), ModuleConfig.class);

        if(moduleManifest != null) {
            module = moduleManifest.getModules().get(0);
        }

        return module;
    }

    private boolean makeOdex(File srcApk,String version) {
        boolean bResult = false;

        File odexDir = ModulePathManager.getInstance().getModuleOdexDir(mModuleID,version);
        if (!odexDir.exists()) {
            odexDir.mkdir();
        }

        File odex = ModulePathManager.getInstance().getModuleOdex(mModuleID,version);

        if (!srcApk.exists()) {
            return  bResult;
        }

        try {
            DexFile.loadDex(srcApk.getAbsolutePath(),odex.getAbsolutePath(),0);
            if (odex.exists()) {
                bResult = true;
            }
        } catch (IOException e) {
            e.printStackTrace();
        }

        return bResult;
    }

}
