package cn.ninegame.genericframework.tools;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.Enumeration;
import java.util.zip.CRC32;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

/**
 * Created by lingcong on 2015/6/12.
 */
public final class ZipUtil {
    private static final int ENDHDR = 22;
    private static final int ENDSIG = 101010256;
    private static final int BUFFER_SIZE = 16384;

    public static long getZipCrc(File apk)
            throws IOException {
        RandomAccessFile raf = new RandomAccessFile(apk, "r");
        try {
            CentralDirectory dir = findCentralDirectory(raf);
            long l = computeCrcOfCentralDir(raf, dir);
            return l;
        } finally {
            raf.close();
        }

    }

    static CentralDirectory findCentralDirectory(RandomAccessFile raf)
            throws IOException {
        long scanOffset = raf.length() - 22L;
        if (scanOffset < 0L) {
            throw new ZipException("File too short to be a zip file: " + raf.length());
        }

        long stopOffset = scanOffset - 65536L;
        if (stopOffset < 0L) {
            stopOffset = 0L;
        }

        int endSig = Integer.reverseBytes(ENDSIG);
        boolean bFound = false;
        do {
            raf.seek(scanOffset);
            if (raf.readInt() == endSig) {
                bFound = true;
                break ;
            }

            scanOffset -= 1L;
        }while (scanOffset >= stopOffset);

        if (!bFound) {
            throw new ZipException("End Of Central Directory signature not found");
        }

        raf.skipBytes(2);
        raf.skipBytes(2);
        raf.skipBytes(2);
        raf.skipBytes(2);
        CentralDirectory dir = new CentralDirectory();
        dir.size = (Integer.reverseBytes(raf.readInt()) & 0xFFFFFFFF);
        dir.offset = (Integer.reverseBytes(raf.readInt()) & 0xFFFFFFFF);
        return dir;
    }

    static long computeCrcOfCentralDir(RandomAccessFile raf, CentralDirectory dir)
            throws IOException {
        CRC32 crc = new CRC32();
        long stillToRead = dir.size;
        raf.seek(dir.offset);
        int length = (int)Math.min(16384L, stillToRead);
        byte[] buffer = new byte[BUFFER_SIZE];
        length = raf.read(buffer, 0, length);
        while (length != -1) {
            crc.update(buffer, 0, length);
            stillToRead -= length;
            if (stillToRead == 0L) {
                break;
            }
            length = (int)Math.min(16384L, stillToRead);
            length = raf.read(buffer, 0, length);
        }
        return crc.getValue();
    }

    static class CentralDirectory {
        long offset;
        long size;
    }

    public static int unZipDir(ZipFile zf, String zipPath, String decompressDir) throws IOException {
        BufferedInputStream bi;
        Enumeration<?> e = zf.entries();
        int zipCountFile = 0;
        while (e.hasMoreElements()) {
            ZipEntry ze2 = (ZipEntry) e.nextElement();
            String entryName = ze2.getName();
            if (entryName.startsWith(zipPath)) {
                entryName = entryName.substring(zipPath.length());
                String path = decompressDir + "/" + entryName;
                if (ze2.isDirectory()) {
                    File decompressDirFile = new File(path);
                    if (!decompressDirFile.exists()) {
                        decompressDirFile.mkdirs();
                    }
                } else {
                    String fileDir = path.substring(0, path.lastIndexOf("/"));
                    File fileDirFile = new File(fileDir);
                    if (!fileDirFile.exists()) {
                        fileDirFile.mkdirs();
                    }
                    BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(decompressDir + "/"
                            + entryName));
                    bi = new BufferedInputStream(zf.getInputStream(ze2));
                    byte[] readContent = new byte[1024];
                    int readCount = bi.read(readContent);
                    while (readCount != -1) {
                        bos.write(readContent, 0, readCount);
                        readCount = bi.read(readContent);
                    }
                    bi.close();
                    bos.close();
                    zipCountFile ++;
                }
            }
        }
        return zipCountFile;
    }

    public static boolean extract(ZipFile zipFile, ZipEntry zipEntry, File extractTo) throws IOException {
        BufferedInputStream in = new BufferedInputStream(zipFile.getInputStream(zipEntry));
        BufferedOutputStream out = null;

        boolean success;
        try {
            out = new BufferedOutputStream(new FileOutputStream(extractTo));
            byte[] buffer = new byte[4096];
            int length = in.read(buffer);
            while (length != -1) {
                out.write(buffer, 0, length);
                length = in.read(buffer);
            }

            success = true;
        }catch (Exception e){
            e.printStackTrace();

            success = false;
        }finally {
            IOUtils.safeClose(out);
            IOUtils.safeClose(in);
        }

        return success;
    }

    /**
     * 从zip包解压出相应的文件
     * @param zipFilePath zip包文件的全路径
     * @param fileName 需要被解压的文件， 多层路径的需要加路径
     * @param extractTo 解压出来的位置
     * @return
     */
    public static boolean unZipFile(String zipFilePath, String fileName, File extractTo) {
        ZipFile zipFile = null;

        try {
            //get the zip file content
            if (zipFilePath != null) {
                zipFile = new ZipFile(zipFilePath);
            }

            if (zipFile != null) {
                ZipEntry zipEntry = zipFile.getEntry(fileName);
                if (zipEntry != null) {
                    return extract(zipFile, zipEntry, extractTo);
                }
            }

        } catch (IOException ex) {
            ex.printStackTrace();
        }

        return false;
    }
}
